class MD5:
    @staticmethod
    def encrypt(string: str):
        import hashlib
        md5 = hashlib.md5(string.encode('utf-8'))
        encrypt_string = md5.hexdigest()
        return encrypt_string



from pysmx.SM2 import generate_keypair as sm2_generate_keypair, Sign as sm2_Sign, Verify as sm2_Verify, \
    Encrypt as sm2_Encrypt, Decrypt as sm2_Decrypt
from pysmx.SM3 import SM3
from pysmx.SM4 import Sm4, ENCRYPT as sm4_ENCRYPT, DECRYPT as sm4_DECRYPT


class SMX:
    @staticmethod
    def generate_sm2_key():
        pk, sk = sm2_generate_keypair()
        return {"pk": pk, "sk": sk}

    @staticmethod
    def generate_sm2_sign(string: str, DA, k, len_para, Hexstr=0, encoding="utf-8"):
        return sm2_Sign(string, DA, k, len_para, Hexstr=Hexstr, encoding=encoding)

    @staticmethod
    def verify_sm2_sign(sign, string: str, PA, len_para, Hexstr=0, encoding="utf-8"):
        return sm2_Verify(sign, string, PA, len_para, Hexstr=Hexstr, encoding=encoding)

    @staticmethod
    def encrypt(string: str, **kwargs):
        encrypt_type = kwargs.get("encrypt_type")
        encryption_algorithm = ["SM2", "SM3", "SM4"]
        encrypt_string = ""
        if encrypt_type in ["SM2", "SM3", "SM4"]:
            if encrypt_type == "SM2":
                encrypt_string = sm2_Encrypt(string.encode(kwargs.get("encoding", "utf-8")), kwargs.get("pk"),
                                             kwargs.get("len_para"),
                                             kwargs.get("Hexstr", 0), kwargs.get("encoding", "utf-8"),
                                             kwargs.get("hash_algorithm", "sm3"))
            elif encrypt_type == "SM3":
                sm3 = SM3()
                sm3.update(string)
                encrypt_string = sm3.hexdigest()
            elif encrypt_type == "SM4":
                sm4 = Sm4()
                sm4.sm4_set_key(kwargs.get("key_data"), kwargs.get("mode"))
                encrypt_string = sm4.sm4_crypt_ecb()
        else:
            raise Exception(
                f"{encrypt_type} is unsupported SMX algorithm,supported algorithm have {encryption_algorithm}")
        return encrypt_string

    @staticmethod
    def decrypt(string: str, **kwargs):
        decrypt_type = kwargs.get("encrypt_type")
        decrypt_algorithm = ["SM2", "SM3", "SM4"]
        decrypt_string = ""
        if decrypt_type in ["SM2", "SM3", "SM4"]:
            if decrypt_type == "SM2":
                decrypt_string = sm2_Decrypt(string, kwargs.get("sk"), kwargs.get("len_para"), kwargs.get("Hexstr", 0),
                                             kwargs.get("encoding", "utf-8"), kwargs.get("hash_algorithm", "sm3"))
            elif decrypt_type == "SM4":
                sm4 = Sm4()
                sm4.sm4_set_key(kwargs.get("key_data"), kwargs.get("mode"))
                decrypt_string = sm4.sm4_crypt_ecb(string)

        else:
            raise Exception(f"{decrypt_type} is unsupported SMX algorithm,supported algorithm have {decrypt_algorithm}")
        return decrypt_string



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2021/12/3 2:19 下午
# Copyright (C) 2021 The lesscode Team
from abc import ABC

from lesscode.db.base_sql_helper import BaseSqlHelper
from lesscode.db.condition_wrapper import ConditionWrapper
from lesscode.db.db_function import DBFunction
from lesscode.db.page import Page


class RelationalDbHelper(BaseSqlHelper, ABC):
    """
    RelationalDbHelper 对关系型数据库操作进行默认实现
    """

    async def insert_data(self, table_name: str, data):
        """
        对于插入接口的整合，不需要考虑是单条插入还是多条插入，自动区分
        :param table_name:
        :param data:
        :return:
        """
        if isinstance(data, list):
            if len(data) == 1:
                return await self.insert_one_data(table_name, data[0])
            else:
                return await self.insert_many_data(table_name, data)
        elif isinstance(data, dict):
            return await self.insert_one_data(table_name, data)

    async def insert_one_data(self, table_name: str, data: dict):
        """
        新增记录插入
        :param table_name: 表名
        :param data: 待插入数据
        :return:
        """
        insert_sql = self.prepare_insert_sql(table_name, data)
        param = []
        for v in data.values():
            if isinstance(v, DBFunction):
                param.append(v.value)
            else:
                param.append(v)
        rs = await self.execute_sql(insert_sql, param)
        return rs

    async def insert_many_data(self, table_name: str, data: list):
        """
        批量插入
        :param table_name:
        :param data:
        :return:
        """
        insert_sql = self.prepare_insert_sql(table_name, data.__getitem__(0))
        return await self.executemany_sql(insert_sql, data)

    async def update_data(self, condition_wrapper: ConditionWrapper, data: dict):
        """
        按条件更新
        :param condition_wrapper: 更新条件
        :param data:更新值
        :return:
        """
        update_sql, values = self.prepare_update_sql(condition_wrapper, data)
        rs = await self.execute_sql(update_sql, values)
        return rs

    async def delete_data(self, condition_wrapper: ConditionWrapper):
        """
        按条件删除
        :param condition_wrapper: 删除条件
        :return:
        """
        delete_sql, values = self.prepare_delete_sql(condition_wrapper)
        rs = await self.execute_sql(delete_sql, values)
        return rs

    async def fetchone_data(self, condition_wrapper: ConditionWrapper):
        """
        查询单条数据
        :param condition_wrapper:  查询条件信息
        :return:
        """
        query_sql, values = self.prepare_query_sql(condition_wrapper)
        return await self.execute_fetchone(query_sql, values)

    async def fetchall_data(self, condition_wrapper: ConditionWrapper):
        """
        查询多条数据
        :param condition_wrapper: 查询条件信息
        :return:
        """
        query_sql, values = self.prepare_query_sql(condition_wrapper)
        return await self.execute_fetchall(query_sql, values)

    async def fetchall_page(self, condition_wrapper: ConditionWrapper, page_num=1, page_size=10):
        """
        分页查询多条数据
        :param condition_wrapper:
        :param page_num: 当前页码
        :param page_size: 每页数量
        :return:
        """
        page_sql, values, count_sql = self.prepare_page_sql(condition_wrapper, page_num, page_size)
        # 查询分页记录
        records = await self.execute_fetchall(page_sql, values)
        # 查询记录数量
        total = await self.execute_fetchone(count_sql, values)
        # 组装成page包装类 最终转为 dict 类型 为转换json
        return Page(records=records, current=page_num, page_size=page_size, total=total.get("total")).__dict__

    def sync_insert_data(self, table_name: str, data):
        """
        对于插入接口的整合，不需要考虑是单条插入还是多条插入，自动区分
        :param table_name:
        :param data:
        :return:
        """
        if isinstance(data, list):
            if len(data) == 1:
                return self.sync_insert_one_data(table_name, data[0])
            else:
                return self.sync_insert_many_data(table_name, data)
        elif isinstance(data, dict):
            return self.sync_insert_one_data(table_name, data)

    def sync_insert_one_data(self, table_name: str, data: dict):
        """
        新增记录插入
        :param table_name: 表名
        :param data: 待插入数据
        :return:
        """
        insert_sql = self.prepare_insert_sql(table_name, data)
        param = []
        for v in data.values():
            if isinstance(v, DBFunction):
                param.append(v.value)
            else:
                param.append(v)
        rs = self.sync_execute_sql(insert_sql, param)
        return rs

    def sync_insert_many_data(self, table_name: str, data: list):
        """
        批量插入
        :param table_name:
        :param data:
        :return:
        """
        insert_sql = self.prepare_insert_sql(table_name, data.__getitem__(0))
        return self.sync_executemany_sql(insert_sql, data)

    def sync_update_data(self, condition_wrapper: ConditionWrapper, data: dict):
        """
        按条件更新
        :param condition_wrapper: 更新条件
        :param data:更新值
        :return:
        """
        update_sql, values = self.prepare_update_sql(condition_wrapper, data)
        rs = self.sync_execute_sql(update_sql, values)
        return rs

    def sync_delete_data(self, condition_wrapper: ConditionWrapper):
        """
        按条件删除
        :param condition_wrapper: 删除条件
        :return:
        """
        delete_sql, values = self.prepare_delete_sql(condition_wrapper)
        rs = self.sync_execute_sql(delete_sql, values)
        return rs

    def sync_fetchone_data(self, condition_wrapper: ConditionWrapper):
        """
        查询单条数据
        :param condition_wrapper:  查询条件信息
        :return:
        """
        query_sql, values = self.prepare_query_sql(condition_wrapper)
        return self.sync_execute_fetchone(query_sql, values)

    def sync_fetchall_data(self, condition_wrapper: ConditionWrapper):
        """
        查询多条数据
        :param condition_wrapper: 查询条件信息
        :return:
        """
        query_sql, values = self.prepare_query_sql(condition_wrapper)
        return self.sync_execute_fetchall(query_sql, values)

    def sync_fetchall_page(self, condition_wrapper: ConditionWrapper, page_num=1, page_size=10):
        """
        分页查询多条数据
        :param condition_wrapper:
        :param page_num: 当前页码
        :param page_size: 每页数量
        :return:
        """
        page_sql, values, count_sql = self.prepare_page_sql(condition_wrapper, page_num, page_size)
        # 查询分页记录
        records = self.sync_execute_fetchall(page_sql, values)
        # 查询记录数量
        total = self.sync_execute_fetchone(count_sql, values)
        # 组装成page包装类 最终转为 dict 类型 为转换json
        return Page(records=records, current=page_num, page_size=page_size, total=total.get("total")).__dict__

    def prepare_insert_sql(self, table_name: str, item: dict):
        """
        组装插入sql
        :param table_name:
        :param item:
        :return:
        """
        # 组装插入数据字段
        column_str = ','.join(item.keys())
        # 依据字段数组织占位符
        # placeholder_str = ','.join('%s' for v in range(len(item)))
        placeholder_items = []
        for v in item.values():
            if isinstance(v, DBFunction):
                placeholder_items.append(v.function_body)
            else:
                placeholder_items.append("%s")
        placeholder_str = ','.join(placeholder_items)
        # 组装可执行SQL
        insert_sql = f"INSERT INTO {table_name} ({column_str}) VALUES ({placeholder_str})"
        return insert_sql

    def prepare_update_sql(self, condition_wrapper: ConditionWrapper, param: dict):
        """
        组装更新sql
        :param condition_wrapper:
        :param param:
        :return:
        """
        # column_sql = ','.join(f"{key}=%s" for key in param.keys())
        value_items = []
        placeholder_items = []
        for key in param.keys():
            v = param.get(key)
            if isinstance(v, DBFunction):
                placeholder_items.append(f"{key}={v.function_body}")
                value_items.append(v.value)
            else:
                placeholder_items.append(f"{key}=%s")
                value_items.append(v)
        column_sql = ','.join(placeholder_items)
        condition_sql, values = self.prepare_condition_sql(condition_wrapper.conditions)
        update_sql = f"UPDATE {condition_wrapper.table} SET {column_sql} WHERE {condition_sql}"
        return update_sql, value_items + values

    def prepare_delete_sql(self, condition_wrapper: ConditionWrapper):
        """
        组装删除SQl
        :param condition_wrapper:
        :return:
        """
        condition_sql, values = self.prepare_condition_sql(condition_wrapper.conditions)
        delete_sql = f"DELETE FROM {condition_wrapper.table} WHERE {condition_sql}"
        return delete_sql, values

    def prepare_condition_sql(self, conditions: list):
        """
        拼接条件SQL 搜集参数信息
        :param conditions: 条件信息
        :return: 条件sql片段, values 对应参数
        """
        connector = "AND"
        condition_sql = ""
        values = []
        for condition in conditions:
            operator, column, value = condition
            if operator == "OR" or operator == "AND":
                connector = operator
                continue
            if operator == "BETWEEN":
                sql_segment = f"{column} BETWEEN %s AND %s"
            elif operator == "NOT_BETWEEN":
                sql_segment = f"{column} NOT BETWEEN %s AND %s"
            elif operator == "LIKE":
                sql_segment = f"{column} LIKE %s"
                value = f"%{value}%"
            elif operator == "NOT_LIKE":
                sql_segment = f"{column} NOT_LIKE %s"
                value = f"%{value}%"
            elif operator == "LIKE_LEFT":
                sql_segment = f"{column} LIKE %s"
                value = f"%{value}"
            elif operator == "LIKE_RIGHT":
                sql_segment = f"{column} LIKE %s"
                value = f"{value}%"
            elif operator == "IS_NULL":
                sql_segment = f"{column} IS NULL "
            elif operator == "IS_NOT_NULL":
                sql_segment = f"{column} IS NOT NULL "
            elif operator == "IN":
                # 依据字段数组织占位符
                placeholder_str = ','.join('%s' for v in range(len(value)))
                sql_segment = f"{column} IN ({placeholder_str})"
            elif operator == "NOT_IN":
                # 依据字段数组织占位符
                placeholder_str = ','.join('%s' for v in range(len(value)))
                sql_segment = f"{column} NOT IN ({placeholder_str})"
            else:
                # 操作符号不需要转码的
                sql_segment = f"{column} {operator} %s"
            if condition_sql:
                condition_sql = f"{condition_sql} {connector} {sql_segment}"
            else:
                condition_sql = sql_segment
            if column is not None:
                if isinstance(value, list):
                    values.extend(value)
                else:
                    values.append(value)
        return condition_sql, values

    def prepare_query_sql(self, condition_wrapper: ConditionWrapper):
        """
                获取完整拼接查询SQL
                :param condition_wrapper: 查询条件对象
                :return:sql 查询sql, values 对应参数
                """
        sql = f"SELECT {','.join(condition_wrapper.column)} FROM {condition_wrapper.table} "
        condition_sql, values = self.prepare_condition_sql(condition_wrapper.conditions)
        # 拼接 条件SQL
        if condition_sql:
            sql += f" WHERE {condition_sql}"
        # 拼接 分组SQL
        if condition_wrapper.group:
            group_sql = ""
            if isinstance(condition_wrapper.group, str):
                group_sql = f" GROUP BY {condition_wrapper.group}"
            elif isinstance(condition_wrapper.group, list):
                group_sql = f" GROUP BY {','.join(condition_wrapper.group)}"
            sql += group_sql
        # 拼接 排序SQL
        if condition_wrapper.order:
            sql += f" ORDER BY {','.join([' '.join(item) for item in condition_wrapper.order])}"
        return sql, values

    def prepare_page_sql(self, condition_wrapper: ConditionWrapper, page_num: int, page_size: int):
        """
        获取分页SQL
        :param condition_wrapper: 查询参数
        :param page_num: 当前页
        :param page_size: 每页记录数
        :return:page_sql 分页sql, values 对应参数, count_sql 统计总数sql
        """
        query_sql, values = self.prepare_query_sql(condition_wrapper)
        cols = []
        for col in condition_wrapper.column:
            col = col.upper().replace("AS", "").strip().split(" ")
            cols.append(col[-1].strip())
        # 组装分页查询语句，需要依据页码计算 起始索引
        page_sql = f"SELECT {','.join(cols)} FROM ({query_sql}) cs OFFSET {Page.skip(page_num, page_size)} LIMIT {page_size}"
        # 组装查询数量语句
        count_sql = f"SELECT count(1) total FROM ({query_sql}) cs"
        return page_sql, values, count_sql



import datetime
import json
from decimal import Decimal

from bson import ObjectId


class JSONEncoder(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, ObjectId):
            return str(o)
        elif isinstance(o, datetime.datetime):
            return str(o)
        elif isinstance(o, bytes):
            return str(o, encoding="utf-8")
        elif isinstance(o, set):
            return list(o)
        elif isinstance(o, Decimal):
            return str(o)
        return json.JSONEncoder.default(self, o)



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2021/11/4 4:39 下午
# Copyright (C) 2021 The lesscode Team

import binascii
import base64
from Crypto.Cipher import AES as _AES


class AES:
    @staticmethod
    def encrypt(key, text):
        """
        :param key: 密钥
        :param text: 需要被加密的数据
        """
        text = str(text)
        aes = _AES.new(AES.add_to_16(key), _AES.MODE_ECB)
        encrypt_aes = aes.encrypt(AES.add_to_16(text))
        encrypted_text = str(base64.encodebytes(encrypt_aes), encoding='utf-8')
        return bytes.decode(binascii.b2a_hex(bytes(encrypted_text, encoding="utf8")))

    @staticmethod
    def decrypt(key, text):
        """
        :param key: 密钥
        :param text: 需要被加密的数据
        """
        text = bytes.decode(binascii.a2b_hex(bytes(text, encoding="utf8")))
        # 密文
        # 初始化加密器
        aes = _AES.new(AES.add_to_16(key), _AES.MODE_ECB)
        # 优先逆向解密base64成bytes
        base64_decrypted = base64.decodebytes(text.encode(encoding='utf-8'))
        # 执行解密密并转码返回str
        decrypted_text = str(aes.decrypt(base64_decrypted), encoding='utf-8').replace('\0', '')
        return decrypted_text

    @staticmethod
    def add_to_16(value):
        """
        :param value: 待处理的数据
        """
        while len(value) % 16 != 0:
            value += '\0'
        return str.encode(value)  # 返回bytes



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2021/11/25 9:52 上午
# Copyright (C) 2021 The lesscode Team

class ConnectionInfo:
    """
    数据库连接信息对象
    """

    def __init__(self, dialect, host, port, password, user=None, db_name=None, name=None, params=None, min_size=3,
                 max_size=10, enable=True, async_enable=True):
        # 数据库dialect类型
        self.dialect = dialect
        # 连接池名称
        if name:
            self.name = name
        else:
            self.name = dialect
        # 主机地址
        self.host = host
        # 端口号
        self.port = port
        # 用户名
        self.user = user
        # 密码
        self.password = password
        # 数据库名称
        self.db_name = db_name
        # 额外参数
        self.params = params
        # 最小数
        self.min_size = min_size
        # 最大数
        self.max_size = max_size
        # 是否启用
        self.enable = enable
        # 是否启用异步
        self.async_enable = async_enable



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2021/11/10 7:52 下午
# Copyright (C) 2021 The lesscode Team


class BusinessException(RuntimeError):
    """
    BusinessException 统一异常类型，增加状态码信息
    """

    def __init__(self, status_code):
        super(BusinessException, self).__init__(status_code[1])
        self.status_code = status_code



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2022/2/17 2:45 下午
# Copyright (C) 2022 The lesscode Team
import psycopg2
from dbutils.pooled_db import PooledDB

from lesscode.db.base_connection_pool import BaseConnectionPool
from aiopg import create_pool


class PostgresqlPool(BaseConnectionPool):

    """
    Postgresql 数据库链接创建类
    """
    async def create_pool(self):
        print("Postgresql create_pool")
        """
        创建postgresql 异步连接池
        :param conn_info: 连接信息
        :return: AsyncConnectionPool
        """
        info = self.conn_info
        if info.async_enable:
            pool = await create_pool(host=info.host, port=info.port, user=info.user,
                                     password=info.password,
                                     database=info.db_name)
            return pool
        else:
            raise NotImplementedError

    def sync_create_pool(self):
        pool = PooledDB(psycopg2, 5, host=self.conn_info.host, port=self.conn_info.port, user=self.conn_info.user,
                        password=self.conn_info.password, database=self.conn_info.db_name)
        return pool



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2022/2/17 2:45 下午
# Copyright (C) 2022 The lesscode Team
from lesscode.db.base_connection_pool import BaseConnectionPool
from pymongo import MongoClient


class MongodbPool(BaseConnectionPool):
    """
    mongodb 数据库链接创建类
    """

    def create_pool(self):
        print("mongodb create_pool")
        """
        创建mongodb 异步连接池
        :param conn_info: 连接信息
        :return:
        """
        info = self.conn_info
        if info.async_enable:
            import motor
            host_str = info.host.split(",")
            hosts = ",".join([f"{host}:{info.port}" for host in host_str])
            conn_info_string = f"mongodb://{info.user}:{info.password}@{hosts}"
            if info.params:
                if info.params == "LDAP":
                    conn_info_string += "/?authMechanism=PLAIN"
                elif info.params == "Password":
                    conn_info_string += "/?authSource=admin"
                elif info.params == "X509":
                    conn_info_string += "/?authMechanism=MONGODB-X509"
            pool = motor.motor_tornado.MotorClient(conn_info_string)
            return pool
        else:
            raise NotImplementedError

    def sync_create_pool(self):
        info = self.conn_info
        if info.async_enable:
            import motor
            host_str = info.host.split(",")
            hosts = ",".join([f"{host}:{info.port}" for host in host_str])
            conn_info_string = f"mongodb://{info.user}:{info.password}@{hosts}"
            if info.params:
                if info.params == "LDAP":
                    conn_info_string += "/?authMechanism=PLAIN"
                elif info.params == "Password":
                    conn_info_string += "/?authSource=admin"
                elif info.params == "X509":
                    conn_info_string += "/?authMechanism=MONGODB-X509"
        pool = MongoClient(conn_info_string)
        return pool



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2022/2/17 2:24 下午
# Copyright (C) 2022 The lesscode Team
from lesscode.db.connection_info import ConnectionInfo


class BaseConnectionPool:

    def __init__(self, conn_info: ConnectionInfo):
        self.conn_info = conn_info

    async def create_pool(self):
        """
        创建连接池
        :return:
        """
        raise NotImplementedError

    def sync_create_pool(self):
        """
        创建连接池
        :return:
        """
        raise NotImplementedError



from threading import Thread


class MyThread(Thread):

    def __init__(self, target, *args, **kwargs):
        super(MyThread, self).__init__()
        self.error_ignore = False
        if "error_ignore" in kwargs:
            self.error_ignore = kwargs.get("error_ignore")
            kwargs.pop("error_ignore")
        self.func = target
        self.args = args
        self._kwargs = kwargs
        self.result = None

    def run(self):
        try:
            self.result = self.func(*self.args, **self._kwargs)
        except Exception as e:
            if not self.error_ignore:
                raise e

    def get_result(self):
        return self.result


def thread_task(task_list: list):
    thread_list = []
    for task in task_list:
        if isinstance(task, list):
            if len(task) == 1:
                thread_list.append(MyThread(target=task[0]))
            elif len(task) == 2:
                if isinstance(task[1], list) or isinstance(task[1], tuple):
                    thread_list.append(MyThread(target=task[0], *task[1]))
                elif isinstance(task[1], dict):
                    thread_list.append(MyThread(target=task[0], **task[1]))
            else:
                raise Exception("task format error")
        else:
            raise Exception("task_list is not list")
    for j in thread_list:
        j.start()
    for t in thread_list:
        t.join()
    return [i.get_result() for i in thread_list]



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2022/2/17 2:45 下午
# Copyright (C) 2022 The lesscode Team
from lesscode.db.base_connection_pool import BaseConnectionPool
from lesscode.db.es.es_request import EsRequest


class EsPool(BaseConnectionPool):
    """
    Elasticsearch 数据库链接创建类
    """

    def create_pool(self):
        """
        创建elasticsearch 异步连接池
        :param conn_info: 连接信息
        :return:
        """
        info = self.conn_info
        if info.async_enable:
            pool = EsRequest(info.host, info.port, info.user, info.password)
            return pool
        else:
            raise NotImplementedError

    def sync_create_pool(self):
        info = self.conn_info
        if info.async_enable:
            pool = EsRequest(info.host, info.port, info.user, info.password)
            return pool
        else:
            raise NotImplementedError



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2021/12/8 10:53 上午
# Copyright (C) 2021 The lesscode Team
import logging

from tornado.options import options
from lesscode.db.base_sql_helper import BaseSqlHelper, echo_sql
from lesscode.db.condition_wrapper import ConditionWrapper
from lesscode.db.page import Page
from lesscode.utils.encryption_algorithm import AES


class ElasticsearchHelper(BaseSqlHelper):
    """
    ElasticsearchHelper  ES数据库操作实现
    """

    async def insert_data(self, table_name: str, data):
        """
        对于插入接口的整合，不需要考虑是单条插入还是多条插入，自动区分
        :param table_name:
        :param data:
        :return:
        """
        if isinstance(data, list):
            if len(data) == 1:
                return await self.insert_one_data(table_name, data[0])
            else:
                return await self.insert_many_data(table_name, data)
        elif isinstance(data, dict):
            return await self.insert_one_data(table_name, data)

    async def insert_one_data(self, table_name: str, data: dict):
        """
        新增记录插入
        :param table_name: 表名
        :param data: 待插入数据
        :return:
        """
        await self.pool.index(index=table_name, body=data)
        return {"rowcount": 1}

    async def insert_many_data(self, table_name: str, data: list):
        """
        批量插入
        :param table_name:
        :param data:
        :return:
        """
        db_str, collection_str = table_name.split(".")
        db = self.pool[db_str]
        collection = db[collection_str]
        rs = await collection.insert_many(data)
        return {"rowcount": len(rs.inserted_ids)}

    async def update_data(self, condition_wrapper: ConditionWrapper, param: dict):
        """
        按条件更新
        :param condition_wrapper: 更新条件
        :param param:更新值
        :return:
        """
        db_str, collection_str = condition_wrapper.table.split(".")
        db = self.pool[db_str]
        collection = db[collection_str]
        filter_items = self.prepare_condition_sql(condition_wrapper.conditions)
        rs = await collection.update_many(filter_items, {"$set": param})
        return {"rowcount": rs.modified_count}

    async def delete_data(self, condition_wrapper: ConditionWrapper):
        """
        按条件删除
        :param condition_wrapper: 删除条件
        :return:
        """
        db_str, collection_str = condition_wrapper.table.split(".")
        db = self.pool[db_str]
        collection = db[collection_str]
        filter_items = self.prepare_condition_sql(condition_wrapper.conditions)
        rs = await collection.delete_many(filter_items)
        return {"rowcount": rs.deleted_count}

    async def fetchone_data(self, condition_wrapper: ConditionWrapper, aes_enable=True, aes_key=None):
        """
        查询单条数据
        :param aes_key: aes加密的key
        :param aes_enable: aes加密开关
        :param condition_wrapper:  查询条件信息
        :return:
        """
        table_str = condition_wrapper.table
        aggregate = self.prepare_query_sql(condition_wrapper)
        return await self.execute_fetchone(table_str, aggregate.get("$match", []), aes_enable=aes_enable,
                                           aes_key=aes_key)

    async def fetchall_data(self, condition_wrapper: ConditionWrapper, aes_enable=True, aes_key=None):
        """
        查询多条数据
        :param aes_key: aes加密的key
        :param aes_enable: aes加密开关
        :param condition_wrapper: 查询条件信息
        :return:
        """
        table_str = condition_wrapper.table
        aggregate = self.prepare_query_sql(condition_wrapper)
        rs = await self.execute_fetchall(table_str, aggregate.get("$match", []), column=aggregate.get("includes", []),
                                         aes_enable=aes_enable, aes_key=aes_key)
        return rs

    async def fetchall_page(self, condition_wrapper: ConditionWrapper, page_num=1, page_size=10, aes_enable=True,
                            aes_key=None):
        """
        分页查询多条数据
        :param aes_key: aes加密的key
        :param aes_enable: aes加密开关
        :param condition_wrapper:
        :param page_num: 当前页码
        :param page_size: 每页数量
        :return:
        """
        table_str = condition_wrapper.table
        aggregate = self.prepare_page_sql(condition_wrapper, page_num, page_size)
        orders = []
        for od in aggregate.get("sort", []):
            for x in od:
                order_type = od.get(x, {}).get("order")
                if order_type in ["ASC", "DESC"]:
                    order_type = order_type.lower()
                else:
                    order_type = "asc"
                orders.append({x: {"order": order_type}})
        records = await self.execute_fetch(table_str, aggregate.get("$match", []), offset=(page_num - 1) * page_size,
                                           size=page_size, column=aggregate.get("includes", []), sort=orders,
                                           aes_enable=aes_enable, aes_key=aes_key)
        total = await self.execute_count(table_str, aggregate.get("$match", []))
        return Page(records=records, current=page_num, page_size=page_size, total=total.get("count", 0)).__dict__

    @echo_sql
    async def execute_fetchone(self, param=None, sql: list = [], column: list = [], aes_enable=True, aes_key=None):
        """
        查询单条数据
        :param aes_key: aes密钥
        :param aes_enable: 是否开启aes加密
        :param column: 返回的字段
        :param sql: 待执行的Sql语句
        :param param: 表名
        :return:
        """

        body = {
            "query": {
                "bool": {
                    "must": sql
                }
            },
            "size": 1
        }
        resp = await self.pool.search(
            index=param,
            body=body,
            _source_includes=column
        )
        hits = resp.get("hits", {}).get("hits", [])
        if hits:
            info = hits[0].get("_source", {})
            info.update({"_id": hits[0].get("_id")})
            if aes_enable:
                key = options.aes_key
                if aes_key:
                    key = aes_key
                info["_id"] = AES.encrypt(key, info.get("_id"))
            return info
        return None

    @echo_sql
    async def execute_fetchall(self, param: str, sql: list = [], column: list = [], scroll: str = "5m",
                               size: int = 100, timeout: str = "3s", sort: list = [], aes_enable=True, aes_key=None):
        """
        查询多条数据
        :param sort: 排序
        :param aes_key: aes密钥
        :param aes_enable: 是否开启aes加密
        :param column: 返回的字段
        :param timeout: 超时时间
        :param size: 数据数量
        :param scroll: 滚动时间
        :param sql: 待执行的Sql语句
        :param param: 表名
        :return:
        """
        body = {
            "query": {
                "bool": {
                    "must": sql
                }
            }
        }
        if sort:
            body["sort"] = sort
        resp = await self.pool.search(
            index=param,
            body=body,
            _source_includes=column,
            scroll=scroll,
            timeout=timeout,
            size=size
        )
        hits = resp.get("hits", {}).get("hits", [])
        records = []
        if hits:
            scroll_id = resp.get("_scroll_id")
            while True:
                rt = await self.pool.scroll(scroll_id=scroll_id, scroll=scroll)
                new_hits = rt.get("hits", {}).get("hits", [])
                if new_hits:
                    hits += new_hits
                else:
                    break
            for x in hits:
                info = x.get("_source", {})
                info.update({"_id": x.get("_id")})
                if aes_enable:
                    key = options.aes_key
                    if aes_key:
                        key = aes_key
                    info["_id"] = AES.encrypt(key, info.get("_id"))
                records.append(info)
        return records

    @echo_sql
    async def execute_fetch(self, param: str, sql: list = [], column: list = None, offset=0,
                            size=100, sort: list = [], track_total_hits=None, aes_enable=True, aes_key=None):
        """
        查询多条数据
        :param sort: 排序
        :param aes_key: aes密钥
        :param aes_enable: 是否开启aes加密
        :param size: 数据量
        :param offset: 偏移量
        :param column: 返回值的字段
        :param sql: 待执行的Sql语句
        :param param: 表名
        :return:
        """
        body = {
            "query": {
                "bool": {
                    "must": sql
                }
            },
            "from": offset,
            "size": size
        }
        if sort:
            body["sort"] = sort
        resp = await self.pool.search(
            index=param,
            body=body,
            track_total_hits=track_total_hits,
            _source_includes=column
        )
        hits = resp.get("hits", {}).get("hits", [])
        records = []
        for x in hits:
            info = x.get("_source", {})
            info.update({"_id": x.get("_id")})
            if aes_enable:
                key = options.aes_key
                if aes_key:
                    key = aes_key
                info["_id"] = AES.encrypt(key, info.get("_id"))
            records.append(info)
        return {
            "data_count": resp["hits"]["total"]["value"],
            "data_list": records,
        }

    @echo_sql
    async def execute_count(self, param: str, sql: list = []):
        """
        查询多条数据
        :param sql: 待执行的Sql语句
        :param param: 表名
        :return:
        """
        body = {
            "query": {
                "bool": {
                    "must": sql
                }
            }
        }
        resp = await self.pool.count(
            index=param,
            body=body
        )
        return resp

    @echo_sql
    async def execute_group(self, param: str, sql: list = [], groups: list = []):
        """
        查询多条数据
        :param groups: 分组字段列表
        :param sql: 待执行的Sql语句
        :param param: 表名
        :return:
        """
        body = {
            "query": {
                "bool": {
                    "must": sql
                }
            }
        }
        key = {}
        if groups:
            for g in groups[::-1]:
                ix = groups.index(g)
                if ix < len(groups):
                    terms = {"terms": {"field": g, "size": 65535}}
                    if key:
                        key = {"aggs": {str(ix): terms, "aggs": key.get("aggs", {})}}
                    else:
                        key = {"aggs": {str(ix): terms}}
        if key:
            body["size"] = 0
            body.update(key)
        resp = await self.pool.search(
            index=param,
            body=body
        )
        return resp

    def sync_insert_data(self, table_name: str, data):
        """
        对于插入接口的整合，不需要考虑是单条插入还是多条插入，自动区分
        :param table_name:
        :param data:
        :return:
        """
        if isinstance(data, list):
            if len(data) == 1:
                return self.sync_insert_one_data(table_name, data[0])
            else:
                return self.sync_insert_many_data(table_name, data)
        elif isinstance(data, dict):
            return self.sync_insert_one_data(table_name, data)

    def sync_insert_one_data(self, table_name: str, data: dict):
        """
        新增记录插入
        :param table_name: 表名
        :param data: 待插入数据
        :return:
        """
        db_str, collection_str = table_name.split(".")
        db = self.pool[db_str]
        collection = db[collection_str]
        collection.insert_one(data)
        return {"rowcount": 1}

    def sync_insert_many_data(self, table_name: str, data: list):
        """
        批量插入
        :param table_name:
        :param data:
        :return:
        """
        db_str, collection_str = table_name.split(".")
        db = self.pool[db_str]
        collection = db[collection_str]
        rs = collection.insert_many(data)
        return {"rowcount": len(rs.inserted_ids)}

    def sync_update_data(self, condition_wrapper: ConditionWrapper, param: dict):
        """
        按条件更新
        :param condition_wrapper: 更新条件
        :param param:更新值
        :return:
        """
        db_str, collection_str = condition_wrapper.table.split(".")
        db = self.pool[db_str]
        collection = db[collection_str]
        filter_items = self.prepare_condition_sql(condition_wrapper.conditions)
        rs = collection.update_many(filter_items, {"$set": param})
        return {"rowcount": rs.modified_count}

    def sync_delete_data(self, condition_wrapper: ConditionWrapper):
        """
        按条件删除
        :param condition_wrapper: 删除条件
        :return:
        """
        db_str, collection_str = condition_wrapper.table.split(".")
        db = self.pool[db_str]
        collection = db[collection_str]
        filter_items = self.prepare_condition_sql(condition_wrapper.conditions)
        rs = collection.delete_many(filter_items)
        return {"rowcount": rs.deleted_count}

    def sync_fetchone_data(self, condition_wrapper: ConditionWrapper, aes_enable=True, aes_key=None):
        """
        查询单条数据
        :param aes_key: aes加密的key
        :param aes_enable: aes加密开关
        :param condition_wrapper:  查询条件信息
        :return:
        """
        table_str = condition_wrapper.table
        aggregate = self.prepare_query_sql(condition_wrapper)
        return self.sync_execute_fetchone(table_str, aggregate.get("$match", []), aes_enable=aes_enable,
                                          aes_key=aes_key)

    def sync_fetchall_data(self, condition_wrapper: ConditionWrapper, aes_enable=True, aes_key=None):
        """
        查询多条数据
        :param aes_key: aes加密的key
        :param aes_enable: aes加密开关
        :param condition_wrapper: 查询条件信息
        :return:
        """
        table_str = condition_wrapper.table
        aggregate = self.prepare_query_sql(condition_wrapper)
        rs = self.sync_execute_fetchall(table_str, aggregate.get("$match", []), column=aggregate.get("includes", []),
                                         aes_enable=aes_enable, aes_key=aes_key)
        return rs

    def sync_fetchall_page(self, condition_wrapper: ConditionWrapper, page_num=1, page_size=10, aes_enable=True,
                            aes_key=None):
        """
        分页查询多条数据
        :param aes_key: aes加密的key
        :param aes_enable: aes加密开关
        :param condition_wrapper:
        :param page_num: 当前页码
        :param page_size: 每页数量
        :return:
        """
        table_str = condition_wrapper.table
        aggregate = self.prepare_page_sql(condition_wrapper, page_num, page_size)
        orders = []
        for od in aggregate.get("sort", []):
            for x in od:
                order_type = od.get(x, {}).get("order")
                if order_type in ["ASC", "DESC"]:
                    order_type = order_type.lower()
                else:
                    order_type = "asc"
                orders.append({x: {"order": order_type}})
        records = self.sync_execute_fetch(table_str, aggregate.get("$match", []), offset=(page_num - 1) * page_size,
                                           size=page_size, column=aggregate.get("includes", []), sort=orders,
                                           aes_enable=aes_enable, aes_key=aes_key)
        total = self.sync_execute_count(table_str, aggregate.get("$match", []))
        return Page(records=records, current=page_num, page_size=page_size, total=total.get("count", 0)).__dict__

    @echo_sql
    def sync_execute_fetchone(self, param=None, sql: list = [], column: list = [], aes_enable=True, aes_key=None):
        """
        查询单条数据
        :param aes_key: aes密钥
        :param aes_enable: 是否开启aes加密
        :param column: 返回的字段
        :param sql: 待执行的Sql语句
        :param param: 表名
        :return:
        """

        body = {
            "query": {
                "bool": {
                    "must": sql
                }
            },
            "size": 1
        }
        resp = self.pool.search(
            index=param,
            body=body,
            _source_includes=column
        )
        hits = resp.get("hits", {}).get("hits", [])
        if hits:
            info = hits[0].get("_source", {})
            info.update({"_id": hits[0].get("_id")})
            if aes_enable:
                key = options.aes_key
                if aes_key:
                    key = aes_key
                info["_id"] = AES.encrypt(key, info.get("_id"))
            return info
        return None

    @echo_sql
    def sync_execute_fetchall(self, param: str, sql: list = [], column: list = [], scroll: str = "5m",
                               size: int = 100, timeout: str = "3s", sort: list = [], aes_enable=True, aes_key=None):
        """
        查询多条数据
        :param sort: 排序
        :param aes_key: aes密钥
        :param aes_enable: 是否开启aes加密
        :param column: 返回的字段
        :param timeout: 超时时间
        :param size: 数据数量
        :param scroll: 滚动时间
        :param sql: 待执行的Sql语句
        :param param: 表名
        :return:
        """
        body = {
            "query": {
                "bool": {
                    "must": sql
                }
            }
        }
        if sort:
            body["sort"] = sort
        resp = self.pool.search(
            index=param,
            body=body,
            _source_includes=column,
            scroll=scroll,
            timeout=timeout,
            size=size
        )
        hits = resp.get("hits", {}).get("hits", [])
        records = []
        if hits:
            scroll_id = resp.get("_scroll_id")
            while True:
                rt = self.pool.scroll(scroll_id=scroll_id, scroll=scroll)
                new_hits = rt.get("hits", {}).get("hits", [])
                if new_hits:
                    hits += new_hits
                else:
                    break
            for x in hits:
                info = x.get("_source", {})
                info.update({"_id": x.get("_id")})
                if aes_enable:
                    key = options.aes_key
                    if aes_key:
                        key = aes_key
                    info["_id"] = AES.encrypt(key, info.get("_id"))
                records.append(info)
        return records

    @echo_sql
    def sync_execute_fetch(self, param: str, sql: list = [], column: list = None, offset=0,
                            size=100, sort: list = [], track_total_hits=None, aes_enable=True, aes_key=None):
        """
        查询多条数据
        :param sort: 排序
        :param aes_key: aes密钥
        :param aes_enable: 是否开启aes加密
        :param size: 数据量
        :param offset: 偏移量
        :param column: 返回值的字段
        :param sql: 待执行的Sql语句
        :param param: 表名
        :return:
        """
        body = {
            "query": {
                "bool": {
                    "must": sql
                }
            },
            "from": offset,
            "size": size
        }
        if sort:
            body["sort"] = sort
        resp = self.pool.search(
            index=param,
            body=body,
            track_total_hits=track_total_hits,
            _source_includes=column
        )
        hits = resp.get("hits", {}).get("hits", [])
        records = []
        for x in hits:
            info = x.get("_source", {})
            info.update({"_id": x.get("_id")})
            if aes_enable:
                key = options.aes_key
                if aes_key:
                    key = aes_key
                info["_id"] = AES.encrypt(key, info.get("_id"))
            records.append(info)
        return {
            "data_count": resp["hits"]["total"]["value"],
            "data_list": records,
        }

    @echo_sql
    def sync_execute_count(self, param: str, sql: list = []):
        """
        查询多条数据
        :param sql: 待执行的Sql语句
        :param param: 表名
        :return:
        """
        body = {
            "query": {
                "bool": {
                    "must": sql
                }
            }
        }
        resp = self.pool.count(
            index=param,
            body=body
        )
        return resp

    @echo_sql
    def sync_execute_group(self, param: str, sql: list = [], groups: list = []):
        """
        查询多条数据
        :param groups: 分组字段列表
        :param sql: 待执行的Sql语句
        :param param: 表名
        :return:
        """
        body = {
            "query": {
                "bool": {
                    "must": sql
                }
            }
        }
        key = {}
        if groups:
            for g in groups[::-1]:
                ix = groups.index(g)
                if ix < len(groups):
                    terms = {"terms": {"field": g, "size": 65535}}
                    if key:
                        key = {"aggs": {str(ix): terms, "aggs": key.get("aggs", {})}}
                    else:
                        key = {"aggs": {str(ix): terms}}
        if key:
            body["size"] = 0
            body.update(key)
        resp = self.pool.search(
            index=param,
            body=body
        )
        return resp

    def prepare_insert_sql(self, table_name: str, item: dict):
        """
        组装插入sql
        :param table_name:
        :param item:
        :return:
        """
        pass

    def prepare_update_sql(self, condition_wrapper: ConditionWrapper, param: dict):
        """
        组装更新sql
        :param condition_wrapper:
        :param param:
        :return:
        """
        pass

    def prepare_delete_sql(self, condition_wrapper: ConditionWrapper):
        """
        组装删除SQl
        :param condition_wrapper:
        :return:
        """
        condition_sql, values = self.prepare_condition_sql(condition_wrapper.conditions)
        delete_sql = f"DELETE FROM {condition_wrapper.table} WHERE {condition_sql}"
        return delete_sql, values

    def prepare_condition_sql(self, conditions: list):
        """
        拼接条件SQL 搜集参数信息
        :param conditions: 条件信息
        :return: 条件sql片段, values 对应参数
        """
        operators = ["AND"]
        if len(conditions) == 0:
            return {}
        filter_item = []
        first_filter_items = {"bool": {"must": []}}
        for condition in conditions:
            # 获取当前过滤条件
            operator, column, value = condition
            if operator in ["OR", "AND"]:
                if operator == "AND":
                    operators.append("AND")
                    first_filter_items = {"bool": {"must": []}}
                elif operator == "OR":
                    operators.append("OR")
                    first_filter_items = {"bool": {"should": []}}
                continue
            if operator == "=":
                condition_item = {"term": {column: value}}
            elif operator == "<>":
                condition_item = {"must_not": {"term": {column: value}}}
            elif operator == ">":
                condition_item = {"range": {column: {"gt": value}}}
            elif operator == ">=":
                condition_item = {"range": {column: {"gte": value}}}
            elif operator == "<":
                condition_item = {"range": {column: {"lt": value}}}
            elif operator == "<=":
                condition_item = {"range": {column: {"lte": value}}}
            elif operator == "BETWEEN":
                # 大于等于小数 小于 大数
                condition_item = {"range": {column: {"gte": value[0], "lt": value[1]}}}
            elif operator == "NOT_BETWEEN":
                # 小于小数  大于等于大数
                condition_item = {"$and": [{column: {"$lt": value[0]}}, {column: {"$gte": value[1]}}]}
            elif operator == "LIKE":
                condition_item = {"match_phrase": {column: value}}
            elif operator == "NOT_LIKE":
                condition_item = {"must_not": {"match_phrase": {column: value}}}
            elif operator == "LIKE_LEFT":
                condition_item = {"prefix": {column: {"value": value}}}
            elif operator == "LIKE_RIGHT":
                condition_item = {column: {"$regex": f".*{value}$", "$options": "$i"}}
            elif operator == "IS_NULL":
                condition_item = {column: None}
            elif operator == "IS_NOT_NULL":
                condition_item = {column: {"$ne": None}}
            elif operator == "IN":
                condition_item = {"terms": {column: value}}
            elif operator == "NOT_IN":
                condition_item = {"must_not": {"terms": {column: value}}}
            else:
                continue
            if operators[-1] == "OR":
                first_filter_items["bool"]["should"].append(condition_item)
            elif operators[-1] == "AND":
                first_filter_items["bool"]["must"].append(condition_item)
            else:
                continue
        filter_item.append(first_filter_items)
        logging.debug(f"Elasticsearch Condition:{filter_item}")
        return filter_item

    def prepare_query_sql(self, condition_wrapper: ConditionWrapper):
        """
        获取完整拼接查询SQL
        :param condition_wrapper: 查询条件对象
        :return:查询对象，分组对象，排序对象
        """
        aggregate = {"includes": condition_wrapper.column}
        # 聚合参数
        if condition_wrapper.conditions:
            filter_items = self.prepare_condition_sql(condition_wrapper.conditions)
            aggregate.update({"$match": filter_items})
        # 拼接 分组SQL
        if condition_wrapper.groups:
            aggregate.update({"$group": condition_wrapper.groups})
        # 拼接 排序SQL
        if condition_wrapper.order:
            order_item = []
            for order in condition_wrapper.order:
                column, op = order
                order_item.append({column: {"order": op, "missing": "_last"}})
            aggregate["sort"] = order_item
        return aggregate

    def prepare_page_sql(self, condition_wrapper: ConditionWrapper, page_num: int, page_size: int):
        """
        获取分页SQL
        :param condition_wrapper: 查询参数
        :param page_num: 当前页
        :param page_size: 每页记录数
        :return:page_sql 分页,  统计总数
        """
        aggregate = self.prepare_query_sql(condition_wrapper)
        if len(aggregate) == 0:
            aggregate = {}
        # 组装查询数量语句
        # 组装分页
        aggregate.update({"from": Page.skip(page_num, page_size)})
        aggregate.update({"size": int(page_size)})
        return aggregate

    async def execute_sql(self, sql: str, param=None):
        pass

    async def executemany_sql(self, sql: str, param=None):
        pass

    def sync_execute_sql(self, sql: str, param=None):
        pass

    def sync_executemany_sql(self, sql: str, param=None):
        pass



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2021/11/4 4:39 下午
# Copyright (C) 2021 The lesscode Team

import inspect
import json
import time
from datetime import datetime
from decimal import Decimal
from typing import Optional, Awaitable, Any

from bson import ObjectId
from tornado import httputil
from tornado.ioloop import IOLoop
from tornado.web import RequestHandler, Application
from lesscode.web.status_code import StatusCode
from lesscode.web.business_exception import BusinessException
from lesscode.web.response_result import ResponseResult
from lesscode.web.router_mapping import RouterMapping


class BaseHandler(RequestHandler):
    """
    BaseHandler 类继承自RequestHandler 用于公共基础功能实现，所有均使用此类为基类，只有使用此类的Handler 会被自动加载
    """

    def __init__(self, application: "Application", request: httputil.HTTPServerRequest, **kwargs):
        """
        初始化方法，完成映射装配
        :param application:
        :param request:
        :param kwargs:
        """
        # 如果想要使用自定义响应，设置self.original为True
        self.start_time = int(round(time.time() * 1000))
        self.end_time = int(round(time.time() * 1000))
        self.original = False
        super().__init__(application, request, **kwargs)

    async def get(self):
        """
        重写父类get请求处理方法，直接调用post方法统一处理
        :return:
        """
        await self.request_handler()

    async def post(self):
        """
        重写父类post请求方法，使其支持通过URL进行方法调用
        改造Tornado 调用处理方式，不在是一个url对应一个类处理，而是使用同实体处理放到一个类中
        通过不同的url指向不同处理方法
        :return:
        """
        await self.request_handler()

    async def options(self):
        """
        解决跨域验证请求
        HTTP的204(No Content)响应, 就表示执行成功, 但是没有数据
        :return:
        """
        self.set_status(204)
        self.finish()

    async def request_handler(self):
        # 通过url路径获取对应的处理方法
        res = [item for item in RouterMapping.instance().handlerMapping
               if item[0] == self.request.path]
        if res:
            # 元组中索引0 存放url，索引 1 存放处理方法对象，此处获取方法对象
            handler_method = res[0][1]
            # 最终整理后的请求参数集合
            params_list = []
            # 获取当前请求的Content-Type
            content_type = self.request.headers.get('Content-Type')
            # 当前请求参数
            arguments = {}
            query_arguments = self.request.query_arguments
            arguments.update(query_arguments)
            if content_type:
                if "application/json" in content_type:
                    request_body = self.request.body
                    if request_body:
                        body_arguments = json.loads(self.request.body.decode())
                        arguments.update(body_arguments)
                else:
                    body_arguments = self.request.body_arguments
                    arguments.update(body_arguments)
            # 获取处理方法的 参数签名
            signature = inspect.signature(handler_method)
            # parameterName 参数名称, parameter 参数对象
            for parameter_name, parameter in signature.parameters.items():
                # 如果参数是self 做特殊处理，传入本身
                if parameter_name == 'self':
                    params_list.append(self)
                else:
                    # 依据参数名称，获取请求参数值
                    argument_value = arguments.get(parameter_name)
                    # 分以下几种情况，第一种情况 未取得请求参数
                    if argument_value is None:
                        # 查看是否有默认值，如果有直接跳过即可
                        if parameter.default is not inspect.Parameter.empty:
                            params_list.append(parameter.default)
                            continue
                        else:
                            # 如果没有有默认值，要抛出异常 提示"请求缺少必要参数"
                            raise BusinessException(StatusCode.REQUIRED_PARAM_IS_EMPTY(parameter_name))
                    # 获取形参类型
                    parameter_type = parameter.annotation
                    # 形参类型为空，尝试获取形参默认值类型
                    if parameter_type is inspect.Parameter.empty:
                        # if parameter.default is inspect.Parameter.empty:
                        #     parameter_type = type(parameter.default)
                        # else:
                        parameter_type = type(parameter.default)
                    # # 获取实参类型
                    # argument_type = type(argument_value)
                    if isinstance(argument_value, list):  # 如果参数是集合类型要进行遍历并进行转码
                        if self.request.method == "GET":
                            if len(argument_value) == 1:
                                # 仅一个 直接存入
                                params_list.append(parse_val(argument_value[0], parameter_type))
                            else:
                                # 多个要使用集合存入
                                params_list.append([parse_val(v, parameter_type) for v in argument_value])
                        else:
                            params_list.append([parse_val(v, parameter_type) for v in argument_value])
                    else:
                        params_list.append(parse_val(argument_value, parameter_type))

            # 判断是否为异步非阻塞方法，true 则直接调用
            if inspect.iscoroutinefunction(handler_method):
                data = await handler_method(*params_list)
            else:
                # 阻塞方法异步调用
                data = await IOLoop.current().run_in_executor(None, handler_method, *params_list)
            self.end_time = int(round(time.time() * 1000))
            self.set_header("Api-Time", f"{self.end_time - self.start_time}ms")
            if not self.original:
                self.write(json.dumps(ResponseResult(data=data), ensure_ascii=False, cls=JSONEncoder))
        else:
            raise BusinessException(StatusCode.RESOURCE_NOT_FOUND)

    def set_default_headers(self):
        """ 设置header参数
            重写父类方法
        :return:
        """
        origin = self.request.headers.get("Origin")
        if origin:
            self.set_header("Access-Control-Allow-Origin", origin)
        else:
            self.set_header("Access-Control-Allow-Origin", "*")
        self.set_header("Request-Id", self.request.headers.get("Request-Id", ""))
        self.set_header("Plugin-Running-Time", self.request.headers.get("Plugin-Running-Time", ""))
        self.set_header("Access-Control-Allow-Headers", "x-requested-with,Authorization,Can-read-cache")
        self.set_header("Access-Control-Allow-Methods", "POST,GET,PUT,DELETE,OPTIONS")
        self.set_header("Access-Control-Expose-Headers", "Access-Token")
        self.set_header("Access-Control-Allow-Credentials", "true")
        self.set_header("Content-Type", "application/json; charset=UTF-8")

    def data_received(self, chunk: bytes) -> Optional[Awaitable[None]]:
        """
        RequestHandler 类中的抽象方法，此处为空实现，解决所有子类都需要继承ABC的问题
        :param chunk:
        :return:
        """
        pass

    def write_error(self, status_code: int, **kwargs) -> None:
        """ 统一处理异常信息返回
        重写父类错误信息函数，业务逻辑代码中引发的异常 在参数对象中exc_info 获取异常信息对象
        :param status_code:
        :param kwargs:
        :return:
        """
        error = kwargs["exc_info"]
        if isinstance(error[1], BusinessException):
            self.set_status(200)
            self.write(json.dumps(ResponseResult(status_code=error[1].status_code), ensure_ascii=False))
        else:
            # for line in traceback.format_exception(*kwargs["exc_info"]):
            #     self.write(line)
            self.write(json.dumps(ResponseResult(StatusCode.INTERNAL_SERVER_ERROR(error[1])), ensure_ascii=False))
        self.finish()

    def _request_summary(self):
        """ 设置请求日志记录格式
        重写父类方法。
        :return:
        """
        return "%s %s %s %s %s" % (
            self.request.method, self.request.path, self.request.arguments, self.request.remote_ip,
            self.request.headers["User-Agent"])

    def get_current_user(self) -> Any:
        """
        获取当前用户信息
        :return:
        """
        user_json = self.request.headers.get("User")
        if user_json:
            # TODO 未做加密
            user_info = json.loads(user_json)
            if user_info and isinstance(user_info, dict):
                user = User()
                for key in user_info:
                    setattr(user, key, user_info[key])
                return user
        return None


class JSONEncoder(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, ObjectId):
            return str(o)
        elif isinstance(o, datetime):
            return str(o)
        elif isinstance(o, bytes):
            return str(o, encoding="utf-8")
        elif isinstance(o, set):
            return list(o)
        elif isinstance(o, Decimal):
            return str(o)
        return json.JSONEncoder.default(self, o)


class User:
    """
    用户对象类
    """

    def __init__(self):
        # '账号id',
        self.id = None
        # 用户名
        self.username = None
        # '密码',
        self.password = None
        # '显示名',
        self.display_name = None
        # 手机号,
        self.phone_no = None
        #  邮箱
        self.email = None
        # 组织机构id',
        self.org_id = None
        # '1正常（激活）；2未激活（管理员新增，首次登录需要改密码）； 3锁定（登录错误次数超限，锁定时长可配置）； 4休眠（长期未登录（字段，时长可配置），定时） 5禁用-账号失效；
        self.account_status = None
        # 角色id集合
        self.roleIds = None


# 解析param
def parse_val(val, val_type):
    # inspect.Parameter.empty <class 'NoneType'>
    # isinstance(parameter.annotation, )
    new_val = val
    if isinstance(val, bytes) or isinstance(val, bytearray):
        new_val = val.decode("utf-8")
    if isinstance(val, list):
        val_list = []
        for item in val:
            val_item = parse_val(item, None)
            val_list.append(val_item)
        if val_type == list:
            return val_list
        else:
            new_val = val_list
    if type(new_val) != val_type:
        if val_type == int:
            return int(new_val)
        elif val_type == str and isinstance(new_val, list):
            return new_val[0]
        elif val_type == dict:
            return json.loads(new_val)
        else:
            return new_val
    return new_val



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2021/11/11 8:06 下午
# Copyright (C) 2021 The lesscode Team
import os
import sys
import logging
import tornado.web
import tornado.ioloop
import tornado.httpserver
from tornado.log import LogFormatter as TornadoLogFormatter  # 避免命名歧义，设置了别名
from tornado.options import options, define
from typing import Optional

from tornado_sqlalchemy import SQLAlchemy

from lesscode.db.init_connection_pool import InitConnectionPool
from lesscode.web.router_mapping import RouterMapping

define("application_name", default=f"{os.path.split(os.path.dirname(sys.argv[0]))[-1]}", type=str, help="配置文件")
define("application_path", default=f"{os.path.dirname(sys.argv[0])}", type=str, help="应用运行根路径")
# 生成秘钥方式 print(base64.b64encode(uuid.uuid4().bytes + uuid.uuid4().bytes).decode())
define("cookie_secret", default="hRHQlJoUQIiNai2MLV9fQV3YDpxWrUHQp/jnfF9riQk=", type=str, help="cookie 秘钥 ")
define("static_path", default=os.path.join(f"{os.path.dirname(sys.argv[0])}", "static"), type=str, help="静态资源目录")
define("aes_key", default="haohaoxuexi", type=str, help="aes加密key")
define("data_server", default="http://127.0.0.1:8901", type=str, help="数据服务")
define("profile", default="profiles.config", type=str, help="配置文件")
define("port", default="8080", type=int, help="服务监听端口")
define("handler_path", default="handlers", type=str, help="处理器文件存储路径")
define("echo_sql", default=True, type=bool, help="是否启用SQL输出")
define("async_enable", default=True, type=bool, help="是否启用异步")
define("max_limit", default=500, type=int, help="单次查询最大数量")
define("eureka_enable", default=False, type=bool, help="是否启用eureka")
define("eureka_server", default="", type=str, help="eureka server地址")
define("eureka_instance_name", default="", type=str, help="eureka实例名称，大写")
define("eureka_instance_ip", default="", type=str, help="eureka实例ip或者域名")
define("eureka_instance_port", default=8901, type=int, help="eureka实例ip或者域名")
define("rms_register_enable", default=False, type=bool, help="是否启动资源注册")
define("rms_register_server", default="http://127.0.0.1:8918", type=str, help="启动资源注册")
define("running_env", default="local", type=str, help="运行环境")
define("db", default=None, type=SQLAlchemy, help="SqlAlchemy")
# 日志初始化配置，配置文件中可修改设置
options.logging = "DEBUG"
options.log_rotate_mode = "time"
options.log_file_prefix = "log"
options.log_rotate_when = "D"
# options.log_rotate_interval,
options.log_file_num_backups = 30
options.async_enable = True
options.eureka_enable = False
options.eureka_server = ""
options.eureka_instance_name = ""
options.eureka_instance_ip = ""
options.eureka_instance_port = 8901
options.rms_register_enable = False
options.rms_register_server = "http://127.0.0.1:8918"
options.running_env = "local"


class LogFormatter(TornadoLogFormatter):
    """
    LogFormatter 类用于日志统一格式化处理
    """

    def __init__(self):
        super(LogFormatter, self).__init__(
            fmt='%(color)s[%(asctime)s] [%(levelname)s] [%(module)s:%(lineno)d]%(end_color)s [%(message)s]',
            datefmt='%Y-%m-%d %H:%M:%S')


# 应用实例
def init_log():
    """
    初始化日志配置，创建控制台日志输出处理类，统一设置格式
    :return:
    """
    console_log = logging.StreamHandler()  # 创建控制台日志输出处理类
    console_log.setLevel(options.logging)  # 设置日志级别
    logging.getLogger().handlers.append(console_log)
    # 为日志处理类指定统一的日志输出格式
    [handler.setFormatter(LogFormatter()) for handler in logging.getLogger().handlers]


def load_profile():
    """
    初始化配置文件，配置文件可通过命令行参数指定 --profile=dev
    :return:
    """
    command_profile_flag = False  # 标识配置文件是否通过命令行参数指定
    # 获取命令行参数，此处仅获取profile配置信息
    profile = [item for item in sys.argv if item.__contains__("--profile=")]
    if profile:  # 如果有指定配置文件，更新参数，如果指定多次，取最后一次
        options.profile = f"profiles.config_{profile[-1].replace('--profile=', '')}"
        command_profile_flag = True
    # 第一次加载配置文件
    try:
        __import__(options.profile)
        # command_profile_flag  是False 表示加载的是默认配置文件，还需要再一次判断默认文件中是否指定了其他配置文件，此判断仅针对默认配置文件，其他配置文件不在进行判断。
        if not command_profile_flag and options.profile != "profiles.config":
            __import__(f"profiles.config_{options.profile}")
    except ModuleNotFoundError:
        # 如果加载失败直接跳过，继续执行，给予WARNING。
        logging.warning(f"配置文件不存在：{options.profile}")


class WebServer:
    """
    Wbe服务对象
    """

    def __init__(self):
        load_profile()  # 1、优先处理加载配置文件，要在命令行解析参数前
        tornado.options.parse_command_line()  # 2、调用tornado的命令行参数解析
        init_log()  # 3、初始化日志配置，要在命令行解析参数后，确保日志参数都已经设置完成。
        # 4、启动时创建数据库连接池
        if options.async_enable:
            tornado.ioloop.IOLoop.current().run_sync(InitConnectionPool.create_pool)
        else:
            InitConnectionPool.sync_create_pool()
        # 5、自动化注册Handler处理类，
        self.autoRegisterHandler(f"{os.path.abspath(os.path.dirname(sys.argv[0]))}/{options.handler_path}",
                                 options.handler_path)
        self.app = tornado.web.Application(
            RouterMapping.instance(),
            cookie_secret=options.cookie_secret,
            static_path=options.static_path,
            db=options.db,
            debug=True if options.logging == "DEBUG" else False  # 依据日志记录级别确认是否开启调试模式
        )
        self.server = tornado.httpserver.HTTPServer(self.app, max_buffer_size=500 * 1024 * 1024, xheaders=True)  # 500M

    def start(self, num_processes: Optional[int] = 1, max_restarts: int = None):
        if options.eureka_enable:
            import py_eureka_client.eureka_client as eureka_client
            eureka_server = options.eureka_server
            eureka_instance_name = options.eureka_instance_name if options.eureka_instance_name \
                else options.application_name
            eureka_instance_ip = options.eureka_instance_ip if options.eureka_instance_ip else "127.0.0.1"
            eureka_instance_port = options.eureka_instance_port if options.eureka_instance_port else options.port
            eureka_client.init(eureka_server=eureka_server,
                               app_name=eureka_instance_name,
                               instance_ip=eureka_instance_ip,
                               instance_port=eureka_instance_port)
        self.server.listen(options.port)
        self.server.start(num_processes, max_restarts)
        logging.info(f"start server : {options.application_name}")
        logging.info(f"start server port : {options.port}")
        try:
            tornado.ioloop.IOLoop.current().start()
        except KeyboardInterrupt:
            pass
        finally:
            # 关掉连接池
            for item in options.database.items():
                if hasattr(item[1][0], "close"):
                    tornado.ioloop.IOLoop.current().run_sync(item[1][0].close)
        return self

    def autoRegisterHandler(self, path, pkg_name):
        """
        动态注册Handler模块
        遍历项目指定包内的Handler，将包内module引入。
        :param path: 项目内Handler的文件路径
        :param pkg_name: 引入模块前缀
        """
        # 首先获取当前目录所有文件及文件夹
        dynamic_handler_names = os.listdir(path)
        for handler_name in dynamic_handler_names:
            # 利用os.path.join()方法获取完整路径
            full_file = os.path.join(path, handler_name)
            # 循环判断每个元素是文件夹还是文件
            if os.path.isdir(full_file):
                # 文件夹递归遍历
                self.autoRegisterHandler(os.path.join(path, handler_name), ".".join([pkg_name, handler_name]))
            elif os.path.isfile(full_file) and handler_name.lower().endswith("handler.py"):
                # 文件，并且为handler结尾，认为是请求处理器，完成动态装载
                __import__("{}.{}".format(pkg_name, handler_name.replace(".py", "")))



import hmac


class HMAC:
    @staticmethod
    def encrypt(string: str, key: str, digestmod="MD5", encoding='utf-8', flag=True):
        h = hmac.new(key.encode(encoding), string.encode(encoding), digestmod=digestmod)
        if flag:
            encrypt_string = h.hexdigest()
        else:
            encrypt_string = h.digest()
        return encrypt_string



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2021/11/18 4:41 下午

import setuptools
from lesscode.version import __version__

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="lesscode-py",
    version=__version__,
    author="Chao.yy",
    author_email="yuyc@ishangqi.com",
    description="lesscode-python 是基于tornado的web开发脚手架项目，该项目初衷为简化开发过程，让研发人员更加关注业务。",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitee.com/yongchao9/lesscode-python",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        # "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    platforms='python',
    install_requires=[
        "tornado>=6.0",
        "tornado-sqlalchemy>=0.7.0",
        "aiomysql>=0.0.22",
        "motor>=2.5.1",
        "elasticsearch==7.15.2",
        "aiohttp>=3.8.1",
        "crypto~=1.4.1",
        "pycryptodome~=3.12.0",
        "aioredis~=2.0.1",
        "DBUtils~=3.0.2",
        "redis>=4.1.4",
        "requests>=2.27.1",
        "neo4j~=5.0.0a1",
        "snowland-smx==0.3.1",
        "py_eureka_client>=0.10.5"
    ]

)
"""
        "aiopg>=1.3.3",
"""
"""
1、打包流程
打包过程中也可以多增加一些额外的操作，减少上传中的错误

# 先升级打包工具
pip install --upgrade setuptools wheel twine

# 打包
python setup.py sdist bdist_wheel

# 检查
twine check dist/*

# 上传pypi
twine upload dist/*
twine upload dist/* -u yuyc -p yu230225
# 安装最新的版本测试
pip install -U lesscode-py -i https://pypi.org/simple
"""



import logging
from datetime import datetime, timedelta

from lesscode.utils.encryption_algorithm import AES
from tornado.options import options


def format_company_id_en(value, route_key):
    key = options.aes_key
    if route_key in ["core.patent", "core.patent_lite"]:
        for data in value.get("tags", {}).get("proposer_type", []):
            if data.get("code"):
                data["code"] = AES.encrypt(key, data["code"])
    elif route_key in ["core.research_institution"]:
        for data in value.get("tags", {}).get("support_unit", []):
            if data.get("id"):
                data["id"] = AES.encrypt(key, data["id"])
    elif route_key in ["core.investment"]:
        for data in value.get("tags", {}).get("invest_company", []):
            if data.get("company_id"):
                data["company_id"] = AES.encrypt(key, data["company_id"])


def format_es_param_result(r, param_list, is_need_decrypt_oralce, is_need_es_score, route_key):
    result_dict = {}
    format_company_id_en(r["_source"], route_key)
    if param_list:
        for key in param_list:
            value = r["_source"]
            key_list = key.split(".")
            for key in key_list:
                if isinstance(value, list):
                    if value:
                        result = []
                        for v in value:
                            if isinstance(v.get(key, None), list):
                                result = result + v[key]
                            else:
                                result.append(v.get(key, None))
                        value = result
                elif isinstance(value, dict):
                    if value:
                        value = value.get(key, None)
                else:
                    pass
            if result_dict.get(key) is None:
                if value:
                    result_dict[key] = value
    else:
        result_dict = r["_source"]
    if is_need_decrypt_oralce:
        r["_id"] = AES.encrypt(options.aes_key, r["_id"])
    result_dict["_id"] = r["_id"]
    if is_need_es_score:
        result_dict["_score"] = r["_score"]
    return result_dict


def es_condition_by_match_phrase(bool_list, column, param, slop=0):
    if param:
        if isinstance(param, list):
            bool_list.append({
                "match_phrase": {
                    column: {
                        "query": param[0],
                        "slop": slop
                    }
                }
            })
        if isinstance(param, str):
            bool_list.append({
                "match_phrase": {
                    column: {
                        "query": param,
                        "slop": slop
                    }
                }
            })


def es_condition_by_match(bool_list, column, param):
    if param:
        if isinstance(param, list):
            bool_list.append({
                "match": {
                    column: {
                        "query": param[0],
                    }
                }
            })
        if isinstance(param, str):
            bool_list.append({
                "match": {
                    column: {
                        "query": param,
                    }
                }
            })


def es_condition_by_not_null(boo_must_list, column, param):
    if param:
        boo_must_list.append({
            "exists": {
                "field": column
            }
        })


def es_condition_by_range(bool_must_list, column, date_list, is_contain_end_date=False):
    if date_list:
        range_dict = {}
        if date_list[0]:
            range_dict["gte"] = date_list[0]
        if len(date_list) == 2 and date_list[1]:
            if "-" in str(date_list[1]) and is_contain_end_date:
                end = str((datetime.strptime(date_list[1], '%Y-%m-%d') + timedelta(days=1)).strftime("%Y-%m-%d"))
                range_dict["lt"] = end
            else:
                end = date_list[1]
                range_dict["lte"] = end
        if range_dict:
            bool_must_list.append({
                "range": {
                    column: range_dict
                }})


def es_condition_by_terms(bool_must_list, column, param_list, is_need_decrypt_oralce=False):
    if param_list:
        param_list = list(filter(None, param_list))
        if is_need_decrypt_oralce:
            for index, id in enumerate(param_list):
                try:
                    param_list[index] = int(AES.decrypt(options.aes_key, id))
                except:
                    logging.error("id转int失败--{}".format(id))
        if param_list:
            bool_must_list.append({
                "terms": {
                    column: param_list
                }})


def es_condition_by_exist(bool_must_list, param, is_exists="是"):
    if param:
        if is_exists == "是" or is_exists == "true" or is_exists == True:
            bool_must_list.append({
                "exists": {
                    "field": param
                }})
        else:
            bool_must_list.append({
                "bool": {
                    "must_not": [
                        {
                            "exists": {
                                "field": param
                            }
                        }
                    ]
                }
            })


def es_condition_by_exist_or_not(bool_must_list, param_dict):
    if param_dict:
        for key in param_dict:
            if param_dict[key] in ["是", "true", True]:
                bool_must_list.append({
                    "exists": {
                        "field": key
                    }})
            else:
                bool_must_list.append({
                    "bool": {
                        "must_not": [
                            {
                                "exists": {
                                    "field": key
                                }
                            }
                        ]
                    }
                })


def es_condition_by_not_in(bool_must_list: list = None, column="", param_list=None):
    if param_list:
        bool_must_list.append({
            "bool": {
                "must_not": {
                    "terms": {
                        column: param_list
                    }}
            }
        })


def es_condition_by_geo_shape(bool_must_list: list = None, column="", polygon=None, geo_type="MultiPolygon",
                              relation="intersects"):
    if polygon:
        bool_must_list.append({
            "geo_shape": {
                column: {
                    "shape": {
                        "type": geo_type,
                        "coordinates": polygon
                    },
                    "relation": relation
                }
            }
        })


def format_bool_must_and_should(bool_must_list, bool_should_more_list):
    if bool_should_more_list:
        for bool_should in bool_should_more_list:
            bool_must_list.append({
                "bool": {
                    "should": bool_should
                }
            })


def format_bool_must_and_must_not(bool_must_list, bool_must_not_more_list):
    if bool_must_not_more_list:
        for bool_must_not in bool_must_not_more_list:
            bool_must_list.append({
                "bool": {
                    "must_not": bool_must_not
                }
            })


def parse_es_sort_list(column, order):
    if order == "asc":
        sort_list = [
            {
                column: {
                    "order": order,
                    "missing": "_last"
                }
            }
        ]
    else:
        sort_list = [
            {
                column: {
                    "order": order,
                    "missing": "_last"
                }
            }
        ]

    return sort_list
